@@ -1,4 +1,4 @@
-/* $Header: /home/cvsroot/NetZ3950/yazwrap/send.c,v 1.6 2003/01/21 16:46:41 mike Exp $ */
+/* $Header: /home/cvsroot/NetZ3950/yazwrap/send.c,v 1.7 2003/06/26 20:34:11 mike Exp $ */
 
 /*
  * yazwrap/send.c -- wrapper functions for Yaz's client API.
@@ -259,6 +259,33 @@ databuf makePresentRequest(databuf referenceId,
 }
 
 
+databuf makeDeleteRSRequest(databuf referenceId,
+			    char *resultSetId,
+			    char **errmsgp)
+{
+    static ODR odr = 0;
+    Z_APDU *apdu;
+    Z_DeleteResultSetRequest *req;
+    Z_ReferenceId zr;
+    Z_ResultSetId *rsList[1];
+    int x;
+
+    if (!prepare_odr(&odr, errmsgp))
+	return nodata((char*) 0);
+    apdu = zget_APDU(odr, Z_APDU_deleteResultSetRequest);
+    req = apdu->u.deleteResultSetRequest;
+
+    req->referenceId = make_ref_id(&zr, referenceId);
+    req->deleteFunction = &x;
+    x = Z_DeleteResultSetRequest_list;
+    req->num_resultSetList = 1;
+    req->resultSetList = &rsList[0];
+    rsList[0] = resultSetId;
+
+    return encode_apdu(odr, apdu, errmsgp);
+}
+
+
 /*
  * If refId is non-null, copy it into the provided buffer, and return
  * a pointer to it; otherwise, return a null pointer.  Either way, the
